#! /usr/bin/env python
# 

# ------------------------------
# 
# La grosse fleme de faire les comptes à chaque fois
# Maintenant c'est toi qui les fait pour moi :D
#
# ------------------------------


# ------------------------------
# Imports
# ------------------------------ 
import csv
import os
import optparse
from pathlib import Path

def account_from_file(file_name):
    """
    Extract account from csv file
    
    :param file_name: csv file name
    :return: list of dictionnaries [{nom:... ,montant:... ,parts:... },...]
    """
    with open(file_name, "r") as f:
        reader = csv.reader(f, delimiter = ",")
        r_comptes =  list(reader)
        header = r_comptes.pop(0) # On récupère le premier élément (les headers)

        comptes = []

        for c in r_comptes: # On parcourt les lignes pour typer les éléments correctement
            compte = {}
            for (i,h) in enumerate(header):
                if "nom" in h.lower():
                    compte["nom"] = c[i].capitalize()
                if "montant" in h.lower():
                    compte["montant"] = eval(c[i])
                if "part" in h.lower():
                    compte["parts"] = int(c[i])
            comptes += [compte]

        return comptes

def forfait(compte, output = print):
    """
    Affiche quelques stats et calcule les crédits et dettes
    
    :param compte: list of dictionnaries [{nom:... ,montant:... ,parts:... },...]
    :return: list of tuples [(name, cred_dept), ...]
    """
    cout_total = sum([c["montant"] for c in compte])
    output("Cout total: {cout_tot}".format(cout_tot = cout_total))

    nbr_jour = sum([c["parts"] for c in compte])
    output("Nombre de parts {njour}".format(njour = nbr_jour))

    cout_jour =  cout_total / nbr_jour
    output("Cout de la part: {cout}".format(cout = round(cout_jour,2)))

    output("\n")

    cred_dept = []
    for pers in compte:
        cred_dept += [[pers["nom"], (pers["montant"] - pers["parts"] * cout_jour)]]
        
    return cred_dept

def normalise(compte):
    """Centre en 0 les comptes"""
    moyenne = sum([c["montant"] for c in compte]) / len(compte)
    compte_normalise = compte

    for (i,n) in enumerate(compte):
        compte_normalise[i]["montant"] = compte[i]["montant"] - moyenne

    return compte_normalise

def affiche_final(donRec, output = print):
    """
    Affiche qui donne quoi à qui à partir de la liste
    
    :param donRec: liste avec qui doit quoi à qui [[qui, àqui, quoi]...]
    """
    for g in donRec:
        output("{don} donne {montant} à {rec}".format(don = g[0], rec = g[1], montant = round(g[2],2)))

def join_comptes(compte1, compte2):
    """
    >>> c1 = [['pop', 3], ['bab', -2]]
    >>> c2 = [['pop', 1], ['nin', -3]]
    >>> join_comptes(c1, c2)
    [('nin', -3), ('pop', 4), ('bab', -2)]
    """
    c_dict1 = {k: v for k, v in compte1}
    c_dict2 = {k: v for k, v in compte2}
    
    joined = {k: c_dict1.get(k, 0) + c_dict2.get(k, 0) for k in set(c_dict1) | set(c_dict2)}
    return [(k, v) for k,v in joined.items()]

def tribut(account, seuil = 0):
    """Tells who owns what to who (recursive algorithm)

    :param seuil: seuil à partir duquel on ne doit plus rien.
    :return: liste des valeurs associées aux débits des comptes après le remboursement
    """
    account.sort(key = lambda s: s[1])
    m = account[0]
    M = account[-1]

    res = ['doneur','receveur',0]

    # Si celui qui doit le plus doit moins que le crédit que celui qui a le plus grand crédit
    # Il va donc tout lui donner
    if  abs(m[1]) <= abs(M[1]):
        M2 = M[1] + m[1]
        # print("{em} donne {em_n} à {eM} donc {em} devient {eM2}".format(em = m[0], em_n = abs(m[1]), eM = M[0] , eM2 = M2)) 
        res = [m[0], M[0], abs(m[1])]
        # On enleve le minimum
        account.remove(m)
        # ON change la valeur du max
        account[-1] = [M[0],M2]
    
    # Sinon il donne juste de quoi compenser
    else:
        m2 = m[1] + M[1]
        # print("{em} donne {eM_n} à {eM} et devient {em2}".format(em = m[0] , em2 = m2 , eM = M[0], eM_n = M[1]))
        res = [m[0], M[0], abs(M[1])]
        # On eleve le max
        account.remove(M)
        # On change la valeur du min
        account[0] = [m[0], m2]

    # Gestion du Seuil
    account = [i for i in account if abs(i[1]) >= seuil]

    if (len(account) > 1):
        #print(account)
        return [res] + tribut(account, seuil=seuil)
    else:
        return [res]

if __name__ == '__main__':
    # Pour analyser les options qu'on lui demande 
    parser = optparse.OptionParser()
    # options proposée
    parser.add_option("-f","--file",action="store", type = "string", dest="file_name", help="Analyse les comptes à partir du fichier donné en argument ou des fichiers du repertoire")
    #parser.add_option("-p","--path",action="store", type = "string", dest="pathname", help="Analyse les comptes à partir de tous les fichiers csv du repertoir (multicompte)")
    parser.add_option("-e","--seuil",action="store", type = "int", dest="seuil",default=0, help="Seuil à partir duquel on concidère qu'il n'est plus nécessaire de payer.")
    # Digestion
    (options, args) = parser.parse_args()

    if options.file_name:
        path = Path(options.file_name)
        assert path.exists()
        if path.is_dir():
            cred_dept = []
            for f in path.glob('*.csv'):
                print(f"Compte: {f.name}")
                compte = account_from_file(f)
                compte_normalise = forfait(compte)
                cred_dept = join_comptes(cred_dept, compte_normalise)

        else:
            compte = account_from_file(options.file_name)
            cred_dept = forfait(compte)

        final = tribut(cred_dept, options.seuil)
        affiche_final(final)


# -----------------------------
# Reglages pour 'vim'
# vim:set autoindent expandtab tabstop=4 shiftwidth=4:
# cursor: 16 del 

